package com.jh.memsource.project.repo;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Repository;
import org.springframework.web.client.HttpStatusCodeException;
import org.springframework.web.client.RestTemplate;

import com.jh.memsource.model.PagedList;
import com.jh.memsource.model.Project;
import com.jh.memsource.model.Settings;

/**
 * This class accesses API at https://cloud.memsource.com/web/docs/api
 * 
 * @author Jan
 *
 */
@Repository
public class ProjectRepository {
	
	private static final Logger LOG = LoggerFactory.getLogger(ProjectRepository.class);

	private RestTemplate restTemplate;
	
	private String apiBaseUrl;
	
	private String loginApiUri;
	
	private String projectApiUri;
	
	private int defaultPageSize;

	/**
	 * Cached auth token
	 */
	private String authToken;
	
	/**
	 * Lists projects
	 * 
	 * @param settings
	 * @param page
	 * @param pageSize
	 * @return
	 */
	public PagedList<Project> listProjects(Settings settings, Integer page, Integer pageSize) {
		if (settings == null) {
			
		}
		if (page == null || page < 0) {
			page = 0;
		}
		if (pageSize == null || pageSize < 1) {
			pageSize = defaultPageSize;
		}
		return fetchProjects(settings, page, pageSize, true);
	}
	
	
	protected PagedList<Project> fetchProjects(Settings settings, int page, int pageSize, boolean retryAuth) {
		synchronized(this) {
			if (authToken == null) {
				authToken = getAuthToken(settings);
			}
		}
		
		try {
			Map<String, Object> params = new HashMap<>();
			params.put("token", authToken);
			params.put("page", page);
			params.put("pageSize", pageSize);
			
			ResponseEntity<ProjectsResponse> response = restTemplate.getForEntity(String.format("%s/%s?token={token}&pageNumber={page}&pageSize={pageSize}", apiBaseUrl, projectApiUri), ProjectsResponse.class, params);
			ProjectsResponse body = response.getBody();

			if (body == null || body.getContent() == null) {
				return new PagedList<Project>();
			}
			
			// map response
			List<ProjectResponse> content = body.getContent();
			List<Project> items = content.stream().map(pr -> new Project(pr.getName(), pr.getStatus(), pr.getSourceLang(), pr.getTargetLangs())).collect(Collectors.toList());
			
			return new PagedList<Project>(items, body.getTotalPages(), pageSize); 
			
		} catch (HttpStatusCodeException e) {
			//handle expired token
			if (HttpStatus.UNAUTHORIZED.equals(e.getStatusCode()) && retryAuth) {
				authToken = null;
				return fetchProjects(settings, page, pageSize, false);
			}
			
			LOG.error("Cannot get list of projects", e);
			throw e;
		}
	}
	

	protected String getAuthToken(Settings settings) {
		ResponseEntity<LoginResponse> response = restTemplate.postForEntity(String.format("%s/%s", apiBaseUrl, loginApiUri), new LoginRequest(settings.getUsername(), settings.getPassword()), LoginResponse.class);
		return response.getBody().getToken();
	}
	
	@Value("${memsource.api.url.base:https://cloud.memsource.com/web}")
	public void setApiBaseUrl(String apiBaseUrl) {
		this.apiBaseUrl = apiBaseUrl;
	}
	
	@Value("${memsource.api.url.login:api2/v1/auth/login}")
	public void setLoginApiUri(String loginApiUri) {
		this.loginApiUri = loginApiUri;
	}

	@Value("${memsource.api.url.project:api2/v1/projects}")
	public void setProjectApiUri(String projectApiUri) {
		this.projectApiUri = projectApiUri;
	}

	@Value("${paging.pageSize:20}")
	public void setDefaultPageSize(int defaultPageSize) {
		this.defaultPageSize = defaultPageSize;
	}


	@Autowired
	public void setRestTemplate(RestTemplate restTemplate) {
		this.restTemplate = restTemplate;
	} 	
	
}
